@@ -14,12 +14,13 @@ from .info import CLIInfoCommand
 from .. import AbstractCLICommand
 from ..logger import cpklogger
 from ...exceptions import NotACPKProjectException
+from ...types import CPKFileMappingTrigger
 
-DEFAULT_NETWORK_MODE = "host"
-LAUNCHER_FMT = "launcher-{launcher}"
+DEFAULT_NETWORK_MODE = "bridge"
 SUPPORTED_SUBCOMMANDS = [
     "attach"
 ]
+TRIGGERS = {CPKFileMappingTrigger.DEFAULT, CPKFileMappingTrigger.RUN_MOUNT}
 
 
 class CLIRunCommand(AbstractCLICommand):
@@ -130,6 +131,12 @@ class CLIRunCommand(AbstractCLICommand):
             action="store_true",
             help="Detach from the container and let it run"
         )
+        parser.add_argument(
+            "--tag",
+            default=None,
+            type=str,
+            help="Custom tag"
+        )
         parser.add_argument(
             "docker_args",
             nargs="*",
@@ -149,7 +156,7 @@ class CLIRunCommand(AbstractCLICommand):
         parsed.workdir = os.path.abspath(parsed.workdir)
 
         # get project
-        project = CPKProject(parsed.workdir)
+        project = CPKProject(parsed.workdir, parsed=parsed)
 
         # show info about project
         CLIInfoCommand.execute(parsed)
@@ -163,14 +170,13 @@ class CLIRunCommand(AbstractCLICommand):
             return False
 
         # check if the index is clean
-        mount_code = parsed.mount is True or isinstance(parsed.mount, str)
-        if project.is_dirty():
-            if mount_code:
-                cpklogger.warning("Your index is not clean (some files are not committed).")
-                cpklogger.warning("If you know what you are doing, use --force (-f) to force.")
-                if not parsed.force:
-                    return False
-                cpklogger.warning("Forced!")
+        mount_source = parsed.mount is True or isinstance(parsed.mount, str)
+        if project.is_dirty() and mount_source:
+            cpklogger.warning("Your index is not clean (some files are not committed).")
+            cpklogger.warning("If you know what you are doing, use --force (-f) to force.")
+            if not parsed.force:
+                return False
+            cpklogger.warning("Forced!")
 
         # sanitize hostname
         if parsed.machine is not None:
@@ -188,6 +194,15 @@ class CLIRunCommand(AbstractCLICommand):
         epoint["MemTotal"] = human_size(epoint["MemTotal"])
         cpklogger.print(DOCKER_INFO.format(**epoint))
 
+        # pick the right architecture if not set
+        if parsed.arch is None:
+            parsed.arch = get_endpoint_architecture(parsed.machine)
+            cpklogger.info(f"Target architecture automatically set to {parsed.arch}.")
+
+        # create defaults
+        image = project.image(parsed.arch)
+        parsed.name = parsed.name or f"cpk-run-{project.name}"
+
         # subcommand "attach"
         if parsed.subcommand == "attach":
             cpklogger.info(f"Attempting to attach to container '{parsed.name}'...")
@@ -206,15 +221,6 @@ class CLIRunCommand(AbstractCLICommand):
             )
             return True
 
-        # pick the right architecture if not set
-        if parsed.arch is None:
-            parsed.arch = get_endpoint_architecture(parsed.machine)
-            cpklogger.info(f"Target architecture automatically set to {parsed.arch}.")
-
-        # create defaults
-        image = project.image(parsed.arch)
-        parsed.name = parsed.name or f"cpk-run-{project.name}"
-
         # print info about multiarch
         msg = "Running an image for {} on {}.".format(parsed.arch, epoint["Architecture"])
         cpklogger.info(msg)
@@ -248,7 +254,7 @@ class CLIRunCommand(AbstractCLICommand):
         # mount code
         mount_option = []
         # mount source code (if requested)
-        if mount_code:
+        if mount_source:
             # (always) mount current project
             projects_to_mount = [parsed.workdir] if parsed.mount is True else []
             # mount secondary projects
@@ -261,24 +267,23 @@ class CLIRunCommand(AbstractCLICommand):
                 # make sure that the project exists
                 if not os.path.isdir(project_path):
                     cpklogger.error('The path "{:s}" is not a CPK project'.format(project_path))
+                    return False
                 # get project info
                 try:
                     proj = CPKProject(project_path)
                 except NotACPKProjectException:
                     cpklogger.error(f"The path '{project_path}' does not contain a CPK project.")
                     return False
-                # TODO: we need to use the triggers instead of this
-                # # get local and remote paths to code and launchfile
-                # local_src, destination_src = proj.code_paths()
-                # local_launch, destination_launch = proj.launch_paths()
-                # # compile mounpoints
-                # mount_option += [
-                #     "-v",
-                #     "{:s}:{:s}".format(os.path.join(project_path, local_src), destination_src),
-                #     "-v",
-                #     "{:s}:{:s}".format(os.path.join(project_path, local_launch),
-                #                        destination_launch),
-                # ]
+                # iterate over list of mappings
+                for mapping in proj.mappings:
+                    if TRIGGERS.intersection(set(mapping.triggers)):
+                        mpoint_source = mapping.source if os.path.isabs(mapping.source) else \
+                            os.path.join(project_path, mapping.source)
+                        mpoint_destination = mapping.destination
+                        # compile mounpoints
+                        mount_option += [
+                            "-v", "{:s}:{:s}".format(mpoint_source, mpoint_destination)
+                        ]
 
         # pulling image (if requested)
         if parsed.pull or parsed.force_pull:
@@ -298,8 +303,10 @@ class CLIRunCommand(AbstractCLICommand):
         if parsed.cmd and parsed.launcher:
             raise ValueError("You cannot use the option --launcher together with --cmd.")
 
+        # environment
+        environment = []
         if parsed.launcher:
-            parsed.cmd = LAUNCHER_FMT.format(launcher=parsed.launcher)
+            environment.extend(["-e", f"CPK_LAUNCHER={parsed.launcher}"])
 
         cmd_option = [] if not parsed.cmd else [parsed.cmd]
         cmd_arguments = (
@@ -319,6 +326,7 @@ class CLIRunCommand(AbstractCLICommand):
             parsed.docker_args += ["--rm"]
         if parsed.detach:
             parsed.docker_args += ["-d"]
+
         # add container name to docker args
         parsed.docker_args += ["--name", parsed.name]
         # escape spaces in arguments
@@ -329,6 +337,7 @@ class CLIRunCommand(AbstractCLICommand):
             + docker_epoint_args
             + ["run", "-it"]
             + module_configuration_args
+            + environment
             + parsed.docker_args
             + mount_option
             + [image]
